@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright (C) 2016, the Pyzo development team
 #
-# Pyzo is distributed under the terms of the (new) BSD License.
+# Pyzo is distributed under the terms of the 2-Clause BSD License.
 # The full license can be found in 'license.txt'.
 
 import sys
@@ -10,24 +10,24 @@ import inspect  # noqa - used in eval()
 
 
 try:
-    import thread # Python 2
+    import thread  # Python 2
 except ImportError:
-    import _thread as thread # Python 3
+    import _thread as thread  # Python 3
 
 
 class PyzoIntrospector(yoton.RepChannel):
     """ This is a RepChannel object that runs a thread to respond to
     requests from the IDE.
     """
-    
-    def _getNameSpace(self, name=''):
+
+    def _getNameSpace(self, name=""):
         """ _getNameSpace(name='')
         
         Get the namespace to apply introspection in.
         If name is given, will find that name. For example sys.stdin.
         
         """
-        
+
         # Get namespace
         NS1 = sys._pyzoInterpreter.locals
         NS2 = sys._pyzoInterpreter.globals
@@ -36,7 +36,7 @@ class PyzoIntrospector(yoton.RepChannel):
         else:
             NS = NS2.copy()
             NS.update(NS1)
-        
+
         # Look up a name?
         if not name:
             return NS
@@ -44,18 +44,18 @@ class PyzoIntrospector(yoton.RepChannel):
             try:
                 # Get object
                 ob = eval(name, None, NS)
-                
+
                 # Get namespace for this object
                 if isinstance(ob, dict):
                     NS = {}
-                    for el in ob :
-                        NS['[' + repr(el) + ']'] = el
+                    for el in ob:
+                        NS["[" + repr(el) + "]"] = ob[el]
                 elif isinstance(ob, (list, tuple)):
                     NS = {}
                     count = -1
                     for el in ob:
                         count += 1
-                        NS['[%i]'%count] = el
+                        NS["[%i]" % count] = el
                 else:
                     keys = dir(ob)
                     NS = {}
@@ -63,15 +63,14 @@ class PyzoIntrospector(yoton.RepChannel):
                         try:
                             NS[key] = getattr(ob, key)
                         except Exception:
-                            NS[key] = '<unknown>'
-                
+                            NS[key] = "<unknown>"
+
                 # Done
                 return NS
-            
+
             except Exception:
                 return {}
-    
-    
+
     def _getSignature(self, objectName):
         """ _getSignature(objectName)
         
@@ -81,107 +80,112 @@ class PyzoIntrospector(yoton.RepChannel):
         the tuple are an empty string.
         
         """
-        
+
         # if a class, get init
         # not if an instance! -> try __call__ instead
         # what about self?
-        
+
         # Get valid object names
-        parts = objectName.rsplit('.')
-        objectNames = ['.'.join(parts[-i:]) for i in range(1,len(parts)+1)]
-        
+        parts = objectName.rsplit(".")
+        objectNames = [".".join(parts[-i:]) for i in range(1, len(parts) + 1)]
+
         # find out what kind of function, or if a function at all!
         NS = self._getNameSpace()
-        fun1 = eval("inspect.isbuiltin(%s)"%(objectName), None, NS)
-        fun2 = eval("inspect.isfunction(%s)"%(objectName), None, NS)
-        fun3 = eval("inspect.ismethod(%s)"%(objectName), None, NS)
+        fun1 = eval("inspect.isbuiltin(%s)" % (objectName), None, NS)
+        fun2 = eval("inspect.isfunction(%s)" % (objectName), None, NS)
+        fun3 = eval("inspect.ismethod(%s)" % (objectName), None, NS)
         fun4 = False
         fun5 = False
         if not (fun1 or fun2 or fun3):
             # Maybe it's a class with an init?
-            if eval("hasattr(%s,'__init__')"%(objectName), None, NS):
+            if eval("hasattr(%s,'__init__')" % (objectName), None, NS):
                 objectName += ".__init__"
-                fun4 = eval("inspect.ismethod(%s)"%(objectName), None, NS)
+                fun4 = eval("inspect.ismethod(%s)" % (objectName), None, NS)
             #  Or a callable object?
-            elif eval("hasattr(%s,'__call__')"%(objectName), None, NS):
+            elif eval("hasattr(%s,'__call__')" % (objectName), None, NS):
                 objectName += ".__call__"
-                fun5 = eval("inspect.ismethod(%s)"%(objectName), None, NS)
-        
+                fun5 = eval("inspect.ismethod(%s)" % (objectName), None, NS)
+
         sigs = ""
         if True:
             # the first line in the docstring is usually the signature
-            tmp = eval("%s.__doc__"%(objectNames[-1]), {}, NS )
-            sigs = ''
+            tmp = eval("%s.__doc__" % (objectNames[-1]), {}, NS)
+            sigs = ""
             if tmp:
                 sigs = tmp.splitlines()[0].strip()
             # Test if doc has signature
             hasSig = False
-            for name in objectNames: # list.append -> L.apend(objec) -- blabla
-                name +="("
+            for name in objectNames:  # list.append -> L.apend(objec) -- blabla
+                name += "("
                 if name in sigs:
                     hasSig = True
             # If not a valid signature, do not bother ...
             if (not hasSig) or (sigs.count("(") != sigs.count(")")):
                 sigs = ""
-        
+
         if fun1 or fun2 or fun3 or fun4 or fun5:
-            
+
             if fun1:
-                kind = 'builtin'
+                kind = "builtin"
             elif fun2:
-                kind = 'function'
+                kind = "function"
             elif fun3:
-                kind = 'method'
+                kind = "method"
             elif fun4:
-                kind = 'class'
+                kind = "class"
             elif fun5:
-                kind = 'callable'
-            
+                kind = "callable"
+
             if not sigs:
                 # Use intospection
-                
-                # collect
+
+                funname = objectName.split(".")[-1]
+
                 try:
-                    tmp = eval("inspect.getargspec(%s)"%(objectName), None, NS)
-                except Exception:  # the above fails on 2.4 (+?) for builtins
-                    tmp = None
-                    kind = ''
-                
-                if tmp is not None:
-                    args, varargs, varkw, defaults = tmp
-                    
-                    # prepare defaults
-                    if defaults is None:
-                        defaults = ()
-                    defaults = list(defaults)
-                    defaults.reverse()
-                    # make list (back to forth)
-                    args2 = []
-                    for i in range(len(args)-fun4):
-                        arg = args.pop()
-                        if i < len(defaults):
-                            args2.insert(0, "%s=%s" % (arg, defaults[i]) )
-                        else:
-                            args2.insert(0, arg )
-                    # append varargs and kwargs
-                    if varargs:
-                        args2.append( "*"+varargs )
-                    if varkw:
-                        args2.append( "**"+varkw )
-                    
-                    # append the lot to our  string
-                    funname = objectName.split('.')[-1]
-                    sigs = "%s(%s)" % ( funname, ", ".join(args2) )
-        
+                    tmp = eval(
+                        "inspect.signature(%s)" % (objectName), None, NS
+                    )  # py3.3
+                    sigs = funname + str(tmp)
+                except Exception:
+                    try:
+                        tmp = eval(
+                            "inspect.getargspec(%s)" % (objectName), None, NS
+                        )  # py2
+                    except Exception:  # the above fails on 2.4 (+?) for builtins
+                        tmp = None
+                        kind = ""
+
+                    if tmp is not None:
+                        args, varargs, varkw, defaults = tmp[:4]
+                        # prepare defaults
+                        if defaults is None:
+                            defaults = ()
+                        defaults = list(defaults)
+                        defaults.reverse()
+                        # make list (back to forth)
+                        args2 = []
+                        for i in range(len(args) - fun4):
+                            arg = args.pop()
+                            if i < len(defaults):
+                                args2.insert(0, "%s=%s" % (arg, defaults[i]))
+                            else:
+                                args2.insert(0, arg)
+                        # append varargs and kwargs
+                        if varargs:
+                            args2.append("*" + varargs)
+                        if varkw:
+                            args2.append("**" + varkw)
+                        # append the lot to our  string
+                        sigs = "%s(%s)" % (funname, ", ".join(args2))
+
         elif sigs:
             kind = "function"
         else:
             sigs = ""
             kind = ""
-        
+
         return sigs, kind
-    
-    
+
     # todo: variant that also says whether it's a property/function/class/other
     def dir(self, objectName):
         """ dir(objectName)
@@ -189,15 +193,15 @@ class PyzoIntrospector(yoton.RepChannel):
         Get list of attributes for the given name.
         
         """
-        #sys.__stdout__.write('handling '+objectName+'\n')
-        #sys.__stdout__.flush()
-        
+        # sys.__stdout__.write('handling '+objectName+'\n')
+        # sys.__stdout__.flush()
+
         # Get namespace
         NS = self._getNameSpace()
-        
+
         # Init names
         names = set()
-        
+
         # Obtain all attributes of the class
         try:
             command = "dir(%s.__class__)" % (objectName)
@@ -206,7 +210,7 @@ class PyzoIntrospector(yoton.RepChannel):
             pass
         else:
             names.update(d)
-        
+
         # Obtain instance attributes
         try:
             command = "%s.__dict__.keys()" % (objectName)
@@ -215,7 +219,7 @@ class PyzoIntrospector(yoton.RepChannel):
             pass
         else:
             names.update(d)
-            
+
         # That should be enough, but in case __dir__ is overloaded,
         # query that as well
         try:
@@ -225,11 +229,10 @@ class PyzoIntrospector(yoton.RepChannel):
             pass
         else:
             names.update(d)
-        
+
         # Respond
         return list(names)
-    
-    
+
     def dir2(self, objectName):
         """ dir2(objectName)
         
@@ -238,60 +241,62 @@ class PyzoIntrospector(yoton.RepChannel):
         
         """
         try:
-            name = ''
+            name = ""
             names = []
+
             def storeInfo(name, val):
                 # Determine type
                 typeName = type(val).__name__
                 # Determine kind
                 kind = typeName
-                if typeName != 'type':
-                    if hasattr(val, '__array__') and hasattr(val, 'dtype'):
-                        kind = 'array'
+                if typeName != "type":
+                    if hasattr(val, "__array__") and hasattr(val, "dtype"):
+                        kind = "array"
                     elif isinstance(val, list):
-                        kind = 'list'
+                        kind = "list"
                     elif isinstance(val, tuple):
-                        kind = 'tuple'
+                        kind = "tuple"
                 # Determine representation
-                if kind == 'array':
-                    tmp = 'x'.join([str(s) for s in val.shape])
+                if kind == "array":
+                    tmp = "x".join([str(s) for s in val.shape])
                     if tmp:
-                        repres = '<array %s %s>' % (tmp, val.dtype.name)
+                        repres = "<array %s %s>" % (tmp, val.dtype.name)
                     elif val.size:
                         tmp = str(float(val))
-                        if 'int' in val.dtype.name:
+                        if "int" in val.dtype.name:
                             tmp = str(int(val))
-                        repres = '<array scalar %s (%s)>' % (val.dtype.name, tmp)
+                        repres = "<array scalar %s (%s)>" % (val.dtype.name, tmp)
                     else:
-                        repres = '<array empty %s>' % (val.dtype.name)
-                elif kind == 'list':
-                    repres = '<list with %i elements>' % len(val)
-                elif kind == 'tuple':
-                    repres = '<tuple with %i elements>' % len(val)
+                        repres = "<array empty %s>" % (val.dtype.name)
+                elif kind == "list":
+                    repres = "<list with %i elements>" % len(val)
+                elif kind == "tuple":
+                    repres = "<tuple with %i elements>" % len(val)
+                elif kind == "dict":
+                    repres = "<dict with {:d} keys>".format(len(val))
                 else:
                     repres = repr(val)
                     if len(repres) > 80:
-                        repres = repres[:77] + '...'
+                        repres = repres[:77] + "..."
                 # Store
                 tmp = (name, typeName, kind, repres)
                 names.append(tmp)
-            
+
             # Get locals
             NS = self._getNameSpace(objectName)
             for name in NS.keys():  # name can be a key in a dict, i.e. not str
-                if hasattr(name, 'startswith') and name.startswith('__'):
+                if hasattr(name, "startswith") and name.startswith("__"):
                     continue
                 try:
                     storeInfo(str(name), NS[name])
                 except Exception:
                     pass
-            
+
             return names
-            
+
         except Exception:
             return []
-    
-    
+
     def signature(self, objectName):
         """ signature(objectName)
         
@@ -303,91 +308,95 @@ class PyzoIntrospector(yoton.RepChannel):
             return text
         except Exception:
             return None
-    
-    
+
     def doc(self, objectName):
         """ doc(objectName)
         
         Get documentation for an object.
         
         """
-        
+
         # Get namespace
         NS = self._getNameSpace()
-        
+
         try:
-            
+
             # collect docstring
-            h_text = ''
+            h_text = ""
             # Try using the class (for properties)
             try:
-                className = eval("%s.__class__.__name__"%(objectName), {}, NS)
-                if '.' in objectName:
-                    tmp = objectName.rsplit('.',1)
-                    tmp[1] += '.'
+                className = eval("%s.__class__.__name__" % (objectName), {}, NS)
+                if "." in objectName:
+                    tmp = objectName.rsplit(".", 1)
+                    tmp[1] += "."
                 else:
-                    tmp = [objectName, '']
-                if className not in ['type', 'module', 'builtin_function_or_method', 'function']:
+                    tmp = [objectName, ""]
+                if className not in [
+                    "type",
+                    "module",
+                    "builtin_function_or_method",
+                    "function",
+                ]:
                     cmd = "%s.__class__.%s__doc__"
-                    h_text = eval(cmd % (tmp[0],tmp[1]), {}, NS)
+                    h_text = eval(cmd % (tmp[0], tmp[1]), {}, NS)
             except Exception:
                 pass
-            
+
             # Normal doc
             if not h_text:
-                h_text = eval("%s.__doc__"%(objectName), {}, NS )
-            
+                h_text = eval("%s.__doc__" % (objectName), {}, NS)
+
             # collect more data
-            h_repr = eval("repr(%s)"%(objectName), {}, NS )
+            h_repr = eval("repr(%s)" % (objectName), {}, NS)
             try:
-                h_class = eval("%s.__class__.__name__"%(objectName), {}, NS )
+                h_class = eval("%s.__class__.__name__" % (objectName), {}, NS)
             except Exception:
                 h_class = "unknown"
-            
+
             # docstring can be None, but should be empty then
             if not h_text:
                 h_text = ""
-            
+
             # get and correct signature
             h_fun, kind = self._getSignature(objectName)
-            if kind == 'builtin' or not h_fun:
+            if kind == "builtin" or not h_fun:
                 h_fun = ""  # signature already in docstring or not available
-            
+
             # cut repr if too long
             if len(h_repr) > 200:
                 h_repr = h_repr[:200] + "..."
             # replace newlines so we can separates the different parts
-            h_repr = h_repr.replace('\n', '\r')
-            
+            h_repr = h_repr.replace("\n", "\r")
+
             # build final text
-            text = '\n'.join([objectName, h_class, h_fun, h_repr, h_text])
-            
+            text = "\n".join([objectName, h_class, h_fun, h_repr, h_text])
+
         except Exception:
             type, value, tb = sys.exc_info()
             del tb
-            text = '\n'.join([objectName, '', '', '', 'No help available. ', str(value)])
+            text = "\n".join(
+                [objectName, "", "", "", "No help available. ", str(value)]
+            )
 
         # Done
         return text
-    
-    
+
     def eval(self, command):
         """ eval(command)
         
         Evaluate a command and return result.
         
         """
-        
+
         # Get namespace
         NS = self._getNameSpace()
-        
+
         try:
             # here globals is None, so we can look into sys, time, etc...
             return eval(command, None, NS)
         except Exception:
-            return 'Error evaluating: ' + command
-    
-    
+            return "Error evaluating: " + command
+
     def interrupt(self, command=None):
         """ interrupt()
         
@@ -403,8 +412,7 @@ class PyzoIntrospector(yoton.RepChannel):
         
         """
         thread.interrupt_main()
-    
-    
+
     def terminate(self, command=None):
         """ terminate()
         
